/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogPPov.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogPPov::TFDialogPPov (HeightField *HF)
	        : GuiDialogOAC ("Terraform Place POV Object Dialog", 
				"POV object placement parameters"), 
		  d_vBoxList (TRUE, 5),
		  d_vBox (FALSE, 5),
		  d_hBox1 (FALSE, 5), 
		  d_hBox2 (FALSE, 5), 
		  d_hBox3 (FALSE, 5),
		  d_hBox4 (FALSE, 5), 
		  d_hBox5 (FALSE, 5), 
		  d_hBox6 (FALSE, 5),
		  d_hBox7 (FALSE, 5) 
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogPPov\n");

	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogPPov::TFDialogPPov");

	d_lblLine1 = new Gtk_Label ("Output Options");
	d_lblLine2 = new Gtk_Label ("Elevation");
	d_lblLine3 = new Gtk_Label ("Placement");
	d_lblLine4 = new Gtk_Label ("HF Scale");
	d_lblLine5 = new Gtk_Label ("Obj Scale");
	d_lblLine6 = new Gtk_Label ("Scale Variance");
	d_lblLine7 = new Gtk_Label ("");
	d_lblOutputFilename = new Gtk_Label ("Filename");
	d_lblObjectName = new Gtk_Label ("POV Object Name");
	d_lblHFScaleX = new Gtk_Label ("X");
	d_lblHFScaleY = new Gtk_Label ("Y");
	d_lblHFScaleZ = new Gtk_Label ("Z");
	d_lblObjMinElv = new Gtk_Label ("Min");
	d_lblObjMaxElv = new Gtk_Label ("Max");
	d_lblPlaceDensity = new Gtk_Label ("Density");
	d_lblPlaceVariance = new Gtk_Label ("Variance");
	d_lblObjScaleX = new Gtk_Label ("X");
	d_lblObjScaleY = new Gtk_Label ("Y");
	d_lblObjScaleZ = new Gtk_Label ("Z");
	d_lblObjScaleXvar = new Gtk_Label ("X");
	d_lblObjScaleYvar = new Gtk_Label ("Y");
	d_lblObjScaleZvar = new Gtk_Label ("Z");

	d_adjHFScaleX = new Gtk_Adjustment (1000, 100, 10000, 1);
	d_adjHFScaleY = new Gtk_Adjustment (250, 50, 5000, 1);
	d_adjHFScaleZ = new Gtk_Adjustment (1000, 100, 10000, 1);
	d_adjObjMinElv = new Gtk_Adjustment (0.66, 0.0, 1.0, 0.01);
	d_adjObjMaxElv = new Gtk_Adjustment (0.95, 0.0, 1.0, 0.01);
	d_adjPlaceDensity = new Gtk_Adjustment (0.5, 0.0, 1.0, 0.01);
	d_adjPlaceVariance = new Gtk_Adjustment (0.5, 0.0, 1.0, 0.01);
	d_adjObjScaleX = new Gtk_Adjustment (5, 1, 25, 1);
	d_adjObjScaleY = new Gtk_Adjustment (20, 1, 250, 1);
	d_adjObjScaleZ = new Gtk_Adjustment (5, 1, 25, 1);
	d_adjObjScaleXvar = new Gtk_Adjustment (0.33, 0.0, 1.0, 0.01);
	d_adjObjScaleYvar = new Gtk_Adjustment (0.33, 0.0, 1.0, 0.01);
	d_adjObjScaleZvar = new Gtk_Adjustment (0.33, 0.0, 1.0, 0.01);

	d_hsHFScaleX = new Gtk_HScale (*d_adjHFScaleX);
	d_hsHFScaleY = new Gtk_HScale (*d_adjHFScaleY);
	d_hsHFScaleZ = new Gtk_HScale (*d_adjHFScaleZ);
	d_hsObjMinElv = new Gtk_HScale (*d_adjObjMinElv);
	d_hsObjMaxElv = new Gtk_HScale (*d_adjObjMaxElv);
	d_hsPlaceDensity = new Gtk_HScale (*d_adjPlaceDensity);
	d_hsPlaceVariance = new Gtk_HScale (*d_adjPlaceVariance);
	d_hsObjScaleX = new Gtk_HScale (*d_adjObjScaleX);
	d_hsObjScaleY = new Gtk_HScale (*d_adjObjScaleY);
	d_hsObjScaleZ = new Gtk_HScale (*d_adjObjScaleZ);
	d_hsObjScaleXvar = new Gtk_HScale (*d_adjObjScaleXvar);
	d_hsObjScaleYvar = new Gtk_HScale (*d_adjObjScaleYvar);
	d_hsObjScaleZvar = new Gtk_HScale (*d_adjObjScaleZvar);

	d_HFO = NULL;
	this->setHF (HF);
	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogPPov::~TFDialogPPov ()
{
	delete d_lblLine1;
	delete d_lblLine2;
	delete d_lblLine3;
	delete d_lblLine4;
	delete d_lblLine5;
	delete d_lblLine6;
	delete d_lblLine7;
	delete d_lblOutputFilename;
	delete d_lblObjectName;
	delete d_lblHFScaleX;
	delete d_lblHFScaleY;
	delete d_lblHFScaleZ;
	delete d_lblObjMinElv;
	delete d_lblObjMaxElv;
	delete d_lblPlaceDensity;
	delete d_lblPlaceVariance;
	delete d_lblObjScaleX;
	delete d_lblObjScaleY;
	delete d_lblObjScaleZ;
	delete d_lblObjScaleXvar;
	delete d_lblObjScaleYvar;
	delete d_lblObjScaleZvar;

	delete d_adjHFScaleX;
	delete d_adjHFScaleY;
	delete d_adjHFScaleZ;
	delete d_adjObjMinElv;
	delete d_adjObjMaxElv;
	delete d_adjPlaceDensity;
	delete d_adjPlaceVariance;
	delete d_adjObjScaleX;
	delete d_adjObjScaleY;
	delete d_adjObjScaleZ;
	delete d_adjObjScaleXvar;
	delete d_adjObjScaleYvar;
	delete d_adjObjScaleZvar;

	delete d_hsHFScaleX;
	delete d_hsHFScaleY;
	delete d_hsHFScaleZ;
	delete d_hsObjMinElv;
	delete d_hsObjMaxElv;
	delete d_hsPlaceDensity;
	delete d_hsPlaceVariance;
	delete d_hsObjScaleX;
	delete d_hsObjScaleY;
	delete d_hsObjScaleZ;
	delete d_hsObjScaleXvar;
	delete d_hsObjScaleYvar;
	delete d_hsObjScaleZvar;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogPPov\n");
}


/*
 *  setHFobjs: set the HF objects we operate on
 */
void TFDialogPPov::setHF (HeightField *HF)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogPPov::setHFobjs");

	if (d_HFO)
                delete d_HFO;

	d_HF = HF;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogPPov::fillVBox ()
{
	int 	s_HBOff = 1,	// hbox offset
		expand=TRUE, 
		fill=FALSE;

	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogPPov called without Frame label",
		"TFDialogPPov::setHF");

	//d_hBox1.pack_start (d_lblLine1, expand, TRUE, s_HBOff);
	//d_lblLine1->show ();
	d_hBox1.pack_start (*d_lblOutputFilename, expand, fill, s_HBOff);
	d_lblOutputFilename->show ();
	d_hBox1.pack_start (d_enOutputFilename, expand, fill+1, s_HBOff);
	d_enOutputFilename.set_text ("hfpovobj.inc");
	d_enOutputFilename.show ();
	d_hBox1.pack_start (*d_lblObjectName, expand, fill, s_HBOff);
	d_lblObjectName->show ();
	d_hBox1.pack_start (d_enObjectName, expand, fill+1, s_HBOff);
	d_enObjectName.set_text ("pov_object");
	d_enObjectName.show ();
	d_vBoxList.pack_start (d_hBox1, TRUE, TRUE, s_VBOff);
	d_hBox1.show ();

	d_hBox2.pack_start (*d_lblLine2, expand, TRUE, s_HBOff);
	d_lblLine2->show ();
	d_hBox2.pack_start (*d_lblObjMinElv, expand, fill, s_HBOff);
	d_lblObjMinElv->show ();
	d_hBox2.pack_start (*d_hsObjMinElv, expand, fill+1, s_HBOff);
	d_hsObjMinElv->set_digits (2);
	d_hsObjMinElv->show ();
	d_hBox2.pack_start (*d_lblObjMaxElv, expand, fill, s_HBOff);
	d_lblObjMaxElv->show ();
	d_hBox2.pack_start (*d_hsObjMaxElv, expand, fill+1, s_HBOff);
	d_hsObjMaxElv->set_digits (2);
	d_hsObjMaxElv->show ();
	d_vBoxList.pack_start (d_hBox2, TRUE, TRUE, s_VBOff);
	d_hBox2.show ();

	d_hBox3.pack_start (*d_lblLine3, expand, TRUE, s_HBOff);
	d_lblLine3->show ();
	d_hBox3.pack_start (*d_lblPlaceDensity, expand, fill, s_HBOff);
	d_lblPlaceDensity->show ();
	d_hBox3.pack_start (*d_hsPlaceDensity, expand, fill+1, s_HBOff);
	d_hsPlaceDensity->set_digits (2);
	d_hsPlaceDensity->show ();
	d_hBox3.pack_start (*d_lblPlaceVariance, expand, fill, s_HBOff);
	d_hsPlaceVariance->set_digits (2);
	d_lblPlaceVariance->show ();
	d_hBox3.pack_start (*d_hsPlaceVariance, expand, fill+1, s_HBOff);
	d_hsPlaceVariance->show ();
	d_vBoxList.pack_start (d_hBox3, TRUE, TRUE, s_VBOff);
	d_hBox3.show ();

	d_hBox4.pack_start (*d_lblLine4, expand, TRUE, s_HBOff);
	d_lblLine4->show ();
	d_hBox4.pack_start (*d_lblHFScaleX, expand, fill, s_HBOff);
	d_lblHFScaleX->show ();
	d_hBox4.pack_start (*d_hsHFScaleX, expand, fill+1, s_HBOff);
	d_hsHFScaleX->set_digits (0);
	d_hsHFScaleX->show ();
	d_hBox4.pack_start (*d_lblHFScaleY, expand, fill, s_HBOff);
	d_hsHFScaleY->set_digits (0);
	d_lblHFScaleY->show ();
	d_hBox4.pack_start (*d_hsHFScaleY, expand, fill+1, s_HBOff);
	d_hsHFScaleY->show ();
	d_hBox4.pack_start (*d_lblHFScaleZ, expand, fill, s_HBOff);
	d_lblHFScaleZ->show ();
	d_hBox4.pack_start (*d_hsHFScaleZ, expand, fill+1, s_HBOff);
	d_hsHFScaleZ->set_digits (0);
	d_hsHFScaleZ->show ();
	d_vBoxList.pack_start (d_hBox4, TRUE, TRUE, s_VBOff);
	d_hBox4.show ();

	d_hBox5.pack_start (*d_lblLine5, expand, TRUE, s_HBOff);
	d_lblLine5->show ();
	d_hBox5.pack_start (*d_lblObjScaleX, expand, fill, s_HBOff);
	d_lblObjScaleX->show ();
	d_hBox5.pack_start (*d_hsObjScaleX, expand, fill+1, s_HBOff);
	d_hsObjScaleX->set_digits (0);
	d_hsObjScaleX->show ();
	d_hBox5.pack_start (*d_lblObjScaleY, expand, fill, s_HBOff);
	d_lblObjScaleY->show ();
	d_hsObjScaleY->set_digits (0);
	d_hBox5.pack_start (*d_hsObjScaleY, expand, fill+1, s_HBOff);
	d_hsObjScaleY->show ();
	d_hBox5.pack_start (*d_lblObjScaleZ, expand, fill, s_HBOff);
	d_hsObjScaleZ->set_digits (0);
	d_lblObjScaleZ->show ();
	d_hBox5.pack_start (*d_hsObjScaleZ, expand, fill+1, s_HBOff);
	d_hsObjScaleZ->show ();
	d_vBoxList.pack_start (d_hBox5, TRUE, TRUE, s_VBOff);
	d_hBox5.show ();

	d_hBox6.pack_start (*d_lblLine6, expand, TRUE, s_HBOff);
	d_lblLine6->show ();
	d_hBox6.pack_start (*d_lblObjScaleXvar, expand, fill, s_HBOff);
	d_lblObjScaleXvar->show ();
	d_hBox6.pack_start (*d_hsObjScaleXvar, expand, fill+1, s_HBOff);
	d_hsObjScaleXvar->show ();
	d_hBox6.pack_start (*d_lblObjScaleYvar, expand, fill, s_HBOff);
	d_lblObjScaleYvar->show ();
	d_hBox6.pack_start (*d_hsObjScaleYvar, expand, fill+1, s_HBOff);
	d_hsObjScaleYvar->show ();
	d_hBox6.pack_start (*d_lblObjScaleZvar, expand, fill, s_HBOff);
	d_lblObjScaleZvar->show ();
	d_hBox6.pack_start (*d_hsObjScaleZvar, expand, fill+1, s_HBOff);
	d_hsObjScaleZvar->show ();
	d_vBoxList.pack_start (d_hBox6, TRUE, TRUE, s_VBOff);
	d_hBox6.show ();

	this->d_baseFrame->add (d_vBoxList);
	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}



void TFDialogPPov::buildDialogWindow ()
{
	this->set_usize (500, 300);
	sprintf (this->d_winTitle, "Place POV Object: %s", d_HF->getName());
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	this->fillActionArea ();
	
	this->show ();
}


/*
 *  buttonCallbackApply: transform the height field
 */
void TFDialogPPov::buttonCallbackApply ()
{
	char 		*outfile, 
			*object;
	int		hfsx, hfsy, hfsz, 
			osx, osy, osz;
	float		floor, ceil,
			density, pvar, 
			osxvar, osyvar, oszvar;

	outfile = const_cast<char*>(d_enOutputFilename.get_text().c_str());
	object = const_cast<char*> (d_enObjectName.get_text().c_str());	
	hfsx = (int)(d_adjHFScaleX->get_value());
	hfsy = (int)(d_adjHFScaleY->get_value());
	hfsz = (int)(d_adjHFScaleZ->get_value());
	floor = d_adjObjMinElv->get_value();
	ceil = d_adjObjMaxElv->get_value();
	density = d_adjPlaceDensity->get_value();
	pvar = d_adjPlaceVariance->get_value();
	osx = (int)(d_adjObjScaleX->get_value());
	osy = (int)(d_adjObjScaleY->get_value());
	osz = (int)(d_adjObjScaleZ->get_value());
	osxvar = d_adjObjScaleXvar->get_value();
	osyvar = d_adjObjScaleYvar->get_value();
	oszvar = d_adjObjScaleZvar->get_value();

	cout << "Names: " << outfile << ", " << object << "\n"; 
	//exit (0);

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogRotate::buttonCallbackApply");
	d_HFO->placePOVObject (outfile, object, 
			hfsx, hfsy, hfsz, 
			floor, ceil, density, pvar,
			osx, osy, osz, osxvar, osyvar, oszvar, 0);
	this->d_applyHit = TRUE;
}


gint TFDialogPPov::delete_event_impl (GdkEventAny *e)
{
	this->hide ();
	return (0);
}

